@@ -96,7 +96,7 @@ BUILTIN_FILE *select_loader(void)
     else if (strstr(install,"text")) loader = &Second;  /* text interface */
     else if (strstr(install,"menu")) loader = &Third;
     else if (strstr(install,"bmp") || bitmap) loader = &Bitmap;   /* menu interface (MDA, EGA, VGA) */
-        
+	
     adapter = get_video();	/* from probe.c */
     if (!nowarn) {
 	if (adapter <= VIDEO_UNKNOWN && (verbose>=2 || loader==&Bitmap))
@@ -196,18 +196,18 @@ static int getval(char **cp, int low, int high, int default_value, int factor)
 void bmp_do_timer(char *cp, MENUTABLE *menu)
 {
     if (!cp) {
-        if (eflag) menu->t_row = menu->t_col = -1;  /* there is none, if not specified during edit */
+	if (eflag) menu->t_row = menu->t_col = -1;  /* there is none, if not specified during edit */
     } else if (strcasecmp(cp,"none")==0) {
-        menu->t_row = menu->t_col = -1;  /* there is none, if specified as "none" */
+	menu->t_row = menu->t_col = -1;  /* there is none, if specified as "none" */
     } else {
 	getval_user = "bmp-timer";
     	menu->t_col = getval(&cp, 1, 76, menu->t_col, 8);
     	menu->t_row = getval(&cp, 1, 30, menu->t_row, 16);
 
     	if (!*cp && !eflag) return;
-    	menu->t_fg = getval(&cp, 0, 15, menu->fg, 0);
-    	menu->t_bg = getval(&cp, 0, 15, eflag?15^menu->t_fg:menu->t_bg, 0);
-    	menu->t_sh = getval(&cp, 0, 15, menu->t_fg, 0);
+    	menu->t_fg = getval(&cp, 0, colormax, menu->fg, 0);
+    	menu->t_bg = getval(&cp, 0, colormax, eflag?colormax^menu->t_fg:menu->t_bg, 0);
+    	menu->t_sh = getval(&cp, 0, colormax, menu->t_fg, 0);
     }
 }
 
@@ -233,8 +233,8 @@ void bmp_do_table(char *cp, MENUTABLE *menu)
     menu->mincol = getval(&cp, 1, menu->maxcol, menu->mincol, 0);
 #endif
     if ((menu->row + menu->maxcol*16 > 480 || 
-         menu->col + (MAX_IMAGE_NAME+1)*8 + (menu->ncol-1)*menu->xpitch > 640)
-          && !nowarn)  
+	 menu->col + (MAX_IMAGE_NAME+1)*8 + (menu->ncol-1)*menu->xpitch > 640)
+	  && !nowarn)  
        fprintf(errstd,"Warning: 'bmp-table' may spill off screen\n");
 }
 
@@ -247,15 +247,15 @@ void bmp_do_colors(char *cp, MENUTABLE *menu)
     }
 
     getval_user = "bmp-colors";
-    menu->fg = getval(&cp, 0, 15, menu->fg, 0);
+    menu->fg = getval(&cp, 0, colormax, menu->fg, 0);
     if (!*cp && !eflag) return;
-    menu->bg = getval(&cp, 0, 15, menu->fg, 0);
-    menu->sh = getval(&cp, 0, 15, menu->fg, 0);
+    menu->bg = getval(&cp, 0, colormax, menu->fg, 0);
+    menu->sh = getval(&cp, 0, colormax, menu->fg, 0);
 
-    menu->h_fg = getval(&cp, 0, 15, menu->h_fg, 0);
+    menu->h_fg = getval(&cp, 0, colormax, menu->h_fg, 0);
     if (!*cp && !eflag) return;
-    menu->h_bg = getval(&cp, 0, 15, menu->h_fg, 0);
-    menu->h_sh = getval(&cp, 0, 15, menu->h_fg, 0);    	
+    menu->h_bg = getval(&cp, 0, colormax, menu->h_fg, 0);
+    menu->h_sh = getval(&cp, 0, colormax, menu->h_fg, 0);    	
 }
 
 void pw_file_update(int passw)
@@ -393,11 +393,11 @@ static void pw_get(char *pass, long crcval[], int option)
     if ((pass2 = strrchr(pass,'/'))) label = pass2+1;
 
     for (walk=pwsave; walk; walk=walk->next) {
-        if (pass == walk->unique ||
-        	(!walk->unique && !strcmp(walk->label,label) && (walk->unique=pass)) ) {
-            memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(long));
-            return;
-        }
+	if (pass == walk->unique ||
+		(!walk->unique && !strcmp(walk->label,label) && (walk->unique=pass)) ) {
+	    memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(long));
+	    return;
+	}
     }
     walk = alloc_t(PASSWORD);
     if (!walk) die("Out of memory");
@@ -440,9 +440,9 @@ static void retrieve_crc(long crcval[])
     else pw_get(cfg_get_strg(cf_options,"password"),crcval,1);
 
     if (verbose >= 2) {
-        printf("Password found is");
-        for (i=0; i<MAX_PW_CRC; i++) printf(" %08lX", crcval[i]);
-        printf("\n");
+	printf("Password found is");
+	for (i=0; i<MAX_PW_CRC; i++) printf(" %08lX", crcval[i]);
+	printf("\n");
     }
 }
 
@@ -523,7 +523,7 @@ static void menu_do_scheme(char *scheme, MENUTABLE *menu)
     int i;
     unsigned char *at;
     /* order of color attributes is:
-          text, hilighted text, border, title
+	  text, hilighted text, border, title
     */
 #define color(c) ((int)strchr(khar,(int)(c))-(int)khar)
     bg = 0;
@@ -546,10 +546,10 @@ static void menu_do_scheme(char *scheme, MENUTABLE *menu)
     }
     /* check on the TEXT color */
     if (menu->at_text == 0) {
-        if (!nowarn)
-            fprintf(errstd, "Warning: menu-scheme \"black on black\" changed to "
-        	"\"white on black\"\n");
-        menu->at_text = 0x07;
+	if (!nowarn)
+	    fprintf(errstd, "Warning: menu-scheme \"black on black\" changed to "
+		"\"white on black\"\n");
+	menu->at_text = 0x07;
     }
     /* check on the HIGHLIGHT color */
     if (menu->at_highlight == 0)  menu->at_highlight = ((menu->at_text<<4)&0x70) | ((menu->at_text>>4)&0x0f);
@@ -573,9 +573,9 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
     static char coms[] = "0123";
     static char parity[] = "NnOoEe";
     static char bps[] = 
-        "110\000150\000300\000600\0001200\0002400\0004800\0009600\000"
-        "19200\00038400\00057600\000115200\000?\000?\000?\000?\000"
-        "56000\000";
+	"110\000150\000300\000600\0001200\0002400\0004800\0009600\000"
+	"19200\00038400\00057600\000115200\000?\000?\000?\000?\000"
+	"56000\000";
     GEOMETRY geo;
     struct stat st;
     int i, speed, bitmap, j, dataend;
@@ -695,12 +695,13 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
    
 	if ( bios_boot == bios_map  &&
 		(bios_passes_dl == DL_GOOD
-/***		 || (do_md_install && !(do_md_install==X_MBR_ONLY )) 	***/
+		 || (do_md_install && !(extra==X_MBR_ONLY ))
 		  ) )
 	    bsect.par_1.prompt |= FLAG_MAP_ON_BOOT;
 	if (bios_boot!=bios_map && !nowarn)
 	    fprintf(errstd, "Warning: The boot sector and map file are on different disks.\n");
-	if ( (bios_map & 0x80) && !do_md_install) /* hard disk & not raid master */
+	if ( (bios_map & 0x80) && !do_md_install &&
+		!cfg_get_flag(cf_options, "static-bios-codes") ) /* hard disk & not raid master */
 	    bsect.par_1.map_serial_no = serial_no[bios_map - 0x80];
 	if (verbose>=2)
 	    printf("bios_boot = 0x%02X  bios_map = 0x%02X  map==boot = %d  map S/N: %08lX\n",
@@ -740,26 +741,26 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	i = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPHEADER)+sizeof(RGB)*16+
 				sizeof(BITMAPLILOHEADER);
 	if (bitmap || st.st_size>i) {
-	    int bits;
+	    int bits=0;
 	    
 	    j = get_std_headers(m_fd, &fhv, &bmhv, &lhv);
 	    if (j<0) die("read %s: %s", message, strerror(errno));
 	    if (j==0 || j>2) { /* definitely a bitmap file */
 		BITMAPHEADER *bmh = &bmhv;
-	        if (verbose >= 3) {
-	            printf("width=%d height=%d planes=%d bits/plane=%d\n",
-	        	(int)bmh->width, (int)bmh->height,
-	        	(int)bmh->numBitPlanes, (int)bmh->numBitsPerPlane);
-	        }
-	        if (bmh->size == sizeof(BITMAPHEADER) &&
-	        	bmh->width==640 && bmh->height==480 && 
-	            	((bits = bmh->numBitPlanes * bmh->numBitsPerPlane) == 4 ||
-	            	  bits == 8) ) {
-	            if (!bitmap) die("Message specifies a bitmap file");
+		if (verbose >= 3) {
+		    printf("width=%d height=%d planes=%d bits/plane=%d\n",
+			(int)bmh->width, (int)bmh->height,
+			(int)bmh->numBitPlanes, (int)bmh->numBitsPerPlane);
+		}
+		if (bmh->size == sizeof(BITMAPHEADER) &&
+			bmh->width==640 && bmh->height==480 && 
+		    	((bits = bmh->numBitPlanes * bmh->numBitsPerPlane) == 4 ||
+		    	  bits == 8) ) {
+		    if (!bitmap) die("Message specifies a bitmap file");
 		    if (bits>4 && adapter<VIDEO_VESA && !nowarn)
 			fprintf(errstd,"Warning: Video adapter does not support VESA BIOS extensions needed for\n"
 				       "  display of 256 colors.  Boot loader will fall back to TEXT only operation.\n");
-	        }
+		}
 		else if (bitmap) die("Unsupported bitmap");
 	    } else if (bitmap) die("Not a bitmap file");
 	}
@@ -797,8 +798,8 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	    walk = bps;
 	    speed = 0;
 	    while (*walk && strncmp(serial+2,walk,(i=strlen(walk)))) {
-	        speed++;
-	        walk += i+1;
+		speed++;
+		walk += i+1;
 	    }
 	    if (!*walk) die("Unsupported baud rate");
 	    param2.ser_param &= ~0xE4;
@@ -824,7 +825,7 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 		}
 	    }
 	    if (verbose>=4) printf("Serial Param = 0x%02X\n", 
-	                                        (int)param2.ser_param);
+						(int)param2.ser_param);
 	}
 	if (delay < 20 && !cfg_get_flag(cf_options,"prompt")) {
 	    fprintf(errstd,"Setting DELAY to 20 (2 seconds)\n");
@@ -939,6 +940,8 @@ static int dev_number(char *dev)
     struct stat st;
 
     if (stat(dev,&st) >= 0) return st.st_rdev;
+    if (verbose >= 1) 
+	printf("Warning: cannot 'stat' device \"%s\"; trying numerical conversion\n", dev);
     return to_number(dev);
 }
 
@@ -950,8 +953,8 @@ static int get_image(char *name,char *label,IMAGE_DESCR *descr)
     unsigned char *uch;
 
     if (!label) {
-        here = strrchr(label = name,'/');
-        if (here) label = here+1;
+	here = strrchr(label = name,'/');
+	if (here) label = here+1;
     }
     if (strchr(label,' ')) die("Image name, label, or alias contains a blank character: '%s'", label);
     if (strlen(label) > MAX_IMAGE_NAME) die("Image name, label, or alias is too long: '%s'",label);
@@ -966,7 +969,7 @@ static int get_image(char *name,char *label,IMAGE_DESCR *descr)
 #endif
 	    die("Duplicate label \"%s\"",label);
 	if ((((descr->flags & FLAG_SINGLE) && strlen(label) == 1) ||
-          (((descrs.d.descr[other].flags) & FLAG_SINGLE) &&
+	  (((descrs.d.descr[other].flags) & FLAG_SINGLE) &&
 	  strlen(descrs.d.descr[other].name) == 1)) &&
 #ifdef LCF_IGNORECASE
 	  toupper(*label) == toupper(*descrs.d.descr[other].name))
@@ -1091,23 +1094,23 @@ if (image) { /* long section specific to 'image=' */
     }
 
     if ((cfg_get_flag(cf_options,"restricted") && 
-             cfg_get_flag(cf_options,"mandatory")) ||
-        (cfg_get_flag(cf_all,"restricted") && 
-             cfg_get_flag(cf_all,"mandatory")))
-         die("MANDATORY and RESTRICTED are mutually exclusive");
+	     cfg_get_flag(cf_options,"mandatory")) ||
+	(cfg_get_flag(cf_all,"restricted") && 
+	     cfg_get_flag(cf_all,"mandatory")))
+	 die("MANDATORY and RESTRICTED are mutually exclusive");
     if (cfg_get_flag(cf_all,"bypass")) {
-        if (cfg_get_flag(cf_all,"mandatory"))
-             die("MANDATORY and BYPASS are mutually exclusive");
-        if (cfg_get_flag(cf_all,"restricted"))
-             die("RESTRICTED and BYPASS are mutually exclusive");
-        if (!cfg_get_strg(cf_options,"password"))
-             die("BYPASS only valid if global PASSWORD is set");
+	if (cfg_get_flag(cf_all,"mandatory"))
+	     die("MANDATORY and BYPASS are mutually exclusive");
+	if (cfg_get_flag(cf_all,"restricted"))
+	     die("RESTRICTED and BYPASS are mutually exclusive");
+	if (!cfg_get_strg(cf_options,"password"))
+	     die("BYPASS only valid if global PASSWORD is set");
     }
     if ((password = cfg_get_strg(cf_all,"password")) && cfg_get_flag(cf_all,"bypass"))
-        die("PASSWORD and BYPASS not valid together");
+	die("PASSWORD and BYPASS not valid together");
     if (password || 
-        ( (password = cfg_get_strg(cf_options,"password")) &&
-          !cfg_get_flag(cf_all,"bypass")  ) ) {
+	( (password = cfg_get_strg(cf_options,"password")) &&
+	  !cfg_get_flag(cf_all,"bypass")  ) ) {
 	if (!*password) {	/* null password triggers interaction */
 	    retrieve_crc((long*)descr->password_crc);
 	} else {
@@ -1146,6 +1149,9 @@ if (image) { /* long section specific to 'image=' */
 	descr->flags |= FLAG_RESTR;
     }
 #endif
+    if (cfg_get_flag(cf_all,"bmp-retain") ||
+      cfg_get_flag(cf_options,"bmp-retain")) descr->flags |= FLAG_RETAIN;
+
     if (cfg_get_flag(cf_all,"single-key") ||
       cfg_get_flag(cf_options,"single-key")) descr->flags |= FLAG_SINGLE;
 
@@ -1261,9 +1267,9 @@ void bsect_update(char *backup_file, int force_backup, int pass)
 static long timestamp = 0;
 
     if (pass>=0) {
-        temp = make_backup(backup_file, force_backup, &bsect_orig,
+	temp = make_backup(backup_file, force_backup, &bsect_orig,
     						boot_dev_nr, "boot sector");
-        if (temp && !timestamp) bsect.par_1.timestamp = timestamp = temp;
+	if (temp && !timestamp) bsect.par_1.timestamp = timestamp = temp;
     }
 
 #ifndef LCF_UNIFY
@@ -1304,7 +1310,7 @@ if (pass>=0) {
 	if (len==0) die ("First stage loader is not relocatable.");
 	
 	space &= 0xFFF0;	/* roll back to paragraph boundary */
-        bsect_wr = bsect_orig;
+	bsect_wr = bsect_orig;
 	memcpy(&bsect_wr.sector[space], &bsect, len);
 	if (space <= 0x80) {
 	    bsect_wr.sector[0] = 0xEB;		/* jmp short */
@@ -1328,7 +1334,15 @@ if (pass>=0) {
 
  /* failsafe check */
 #if 1
-    if (verbose >= 3) printf("Failsafe check:  boot_dev_nr = 0x%04x 0x%04x\n", boot_dev_nr, has_partitions(boot_dev_nr));
+    if (verbose >= 3) {
+	printf("Failsafe check:  boot_dev_nr = 0x%04x 0x%04x\n", boot_dev_nr, has_partitions(boot_dev_nr));
+	/*** if (do_md_install) ***/ {
+	    printf("map==boot = %d    map s/n = %08lX\n",
+		!!(bsect_wr.par_1.prompt & FLAG_MAP_ON_BOOT),
+		bsect_wr.par_1.map_serial_no
+	    );
+	}
+    }
 #endif
     if (
       has_partitions(boot_dev_nr) &&
@@ -1507,15 +1521,17 @@ void bsect_raid_update(char *boot_dev, unsigned long raid_offset,
 
     if (pass != 0) {    
 	bsect_save = bsect;			/* save the generated boot sector */
-        open_bsect(boot_dev);
-        memcpy(&bsect, &bsect_save, MAX_BOOT_SIZE);	/* update the subject boot sector */
-        bsect.par_1.raid_offset = raid_offset;	/* put in the new partition offset */
-        bsect.par_1.prompt &= mask;		/* clear all RAID flags */
-        bsect.par_1.prompt |= raid_flags;	/* update the raid flags */
+	open_bsect(boot_dev);
+	memcpy(&bsect, &bsect_save, MAX_BOOT_SIZE);	/* update the subject boot sector */
+	bsect.par_1.raid_offset = raid_offset;	/* put in the new partition offset */
+	bsect.par_1.prompt &= mask;		/* clear all RAID flags */
+	bsect.par_1.prompt |= raid_flags;	/* update the raid flags */
 
-        bios = (raid_flags&FLAG_RAID_DEFEAT) ? bios_map : bios_boot;
-        if (verbose>=2) printf("Using s/n from device 0x%02X\n", bios);
-	bsect.par_1.map_serial_no = serial_no[bios - 0x80];
+	bios = (raid_flags&FLAG_RAID_DEFEAT) ? bios_map : bios_boot;
+	if (!cfg_get_flag(cf_options, "static-bios-codes")) {
+	    if (verbose>=2) printf("Using s/n from device 0x%02X\n", bios);
+	    bsect.par_1.map_serial_no = serial_no[bios - 0x80];
+	}
 
 #ifdef LCF_FIRST6
 /* lines added 22.5.7 */
@@ -1527,7 +1543,7 @@ void bsect_raid_update(char *boot_dev, unsigned long raid_offset,
 #endif
 /* ************************************ */
 
-        *(unsigned short *) &bsect.sector[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
+	*(unsigned short *) &bsect.sector[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
     }
     
     if (pass<0) pass = -pass;
