@@ -49,7 +49,7 @@ void bsect_update(char *backup_file, int force_backup, int pass);
 /* Updates the boot sector and the map file. */
 
 void bsect_raid_update(char *boot_dev, unsigned long raid_offset, 
-	char *backup_file, int force_backup, int pass);
+	char *backup_file, int force_backup, int pass, int mask);
 
 /* Update the boot sector and the map file, with RAID considerations */
 
