@@ -59,10 +59,15 @@ CONFIG cf_identify[] = {
 
 CONFIG cf_options[] = {
   { cft_strg, "append",		NULL,		NULL,NULL },
+/*#ifdef LCF_ATARAID*/
+#if 0
+  { cft_flag, "ataraid",	NULL,		NULL,NULL },
+#endif
   { cft_strg, "backup",		NULL,		NULL,NULL },
   { cft_strg, "bios-passes-dl",	NULL,		NULL,NULL },
   { cft_strg, "bitmap",		NULL,		NULL,NULL },
   { cft_strg, "bmp-colors",	NULL,		NULL,NULL },
+  { cft_flag, "bmp-retain",	NULL,		NULL,NULL },
   { cft_strg, "bmp-table",	NULL,		NULL,NULL },
   { cft_strg, "bmp-timer",	NULL,		NULL,NULL },
   { cft_strg, "boot",		NULL,		NULL,NULL },
@@ -105,6 +110,7 @@ CONFIG cf_options[] = {
   { cft_strg, "root",		NULL,		NULL,NULL },
   { cft_strg, "serial",		NULL,		NULL,NULL },
   { cft_flag, "single-key",	NULL,		NULL,NULL },
+  { cft_flag, "static-bios-codes",	NULL,		NULL,NULL },
   { cft_flag, "suppress-boot-time-BIOS-data",	NULL,	NULL,NULL },
   { cft_strg, "timeout",	NULL,		NULL,NULL },
   { cft_strg, "verbose",	NULL,		NULL,NULL },
@@ -114,6 +120,7 @@ CONFIG cf_options[] = {
 
 CONFIG cf_all[] = {
   { cft_strg, "alias",		NULL,		NULL,NULL },
+  { cft_flag, "bmp-retain",	NULL,		NULL,NULL },
   { cft_flag, "bypass",		NULL,		NULL,NULL },
   { cft_strg, "fallback",	NULL,		NULL,NULL },
   { cft_strg, "label",		NULL,		NULL,NULL },
@@ -157,6 +164,7 @@ CONFIG cf_disk[] = {
   { cft_strg, "cylinders",	NULL,		NULL,NULL },
   { cft_strg, "heads",		NULL,		NULL,NULL },
   { cft_flag, "inaccessible",	NULL,		NULL,NULL },
+  { cft_strg, "max-partitions",	NULL,		NULL,NULL },
   { cft_strg, "sectors",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
