@@ -20,12 +20,15 @@ source directory.
 #include "common.h"
 
 #ifndef SHS_MAIN
-EXTRA extra;
+LILO_EXTRA extra;
+char *identify = NULL;
 int verbose = 0,test = 0,compact = 0,linear = 0, raid_flags = 0, zflag = 0,
-      ireloc = 0, force_fs = 0, extended_pt = 0,
+      ireloc = 0, force_fs = 0, extended_pt = 0, query = 0,
       nowarn = 0, lba32 = 0, autoauto = 0, passw = 0, geometric = 0, eflag = 0;
+int colormax = 15;
 DL_BIOS bios_passes_dl = DL_NOT_SET;
 
+FILE *pp_fd = NULL;
 int bios_boot, bios_map;
 unsigned short drv_map[DRVMAP_SIZE+1]; /* fixup maps ... */
 int curr_drv_map;
