@@ -9,6 +9,7 @@ source directory.
 
 */
 #define DEBUG 0
+#define ATAKEYWORD 0
 
 #define _GNU_SOURCE
 #include <unistd.h>
@@ -33,7 +34,7 @@ source directory.
 #include "partition.h"
 #include "cfg.h"
 #include "probe.h"
-
+#include "ata-u.h"
 
 typedef struct _cache_entry {
     const char *name;
@@ -338,8 +339,10 @@ void preload_dev_cache(void)
     
     cache_ide("/dev/hdl", MAJOR_IDE6);
     cache_ide("/dev/hdk", MAJOR_IDE6);
+#ifdef MAJOR_IDE5
     cache_ide("/dev/hdj", MAJOR_IDE5);
     cache_ide("/dev/hdi", MAJOR_IDE5);
+#endif
 #endif
     
     for (i = MAX; i >= 0; i--) {
@@ -508,12 +511,12 @@ int serial_valid(unsigned long serial, int disk_bios)
     if (serial == -1 || serial == 0) return 0;
     return 1;
 #else
-/* if serial is replicated characters, it is invalid */
+/* if ID is replicated characters, it is invalid */
 /*	Examples of invalid Volume ID's are:
 		00000000
 		6C6C6C6C
 		FFFFFFFF
-BUT: any serial number (except 0 or -1) is valid on drive C:
+BUT: any Volume ID (except 0 or -1) is valid on drive C:
  - - - - - - - - - - - - - - - - - - - - - - - - - - */
     unsigned long temp;
     
@@ -562,9 +565,9 @@ unsigned long serial_no[MAX_BIOS_DEVICES];
 static int device_code[MAX_BIOS_DEVICES];
 
 /*  register references to various bios devices
- *  compiles the list of volume serial numbers
- *    returns volume serial number on success
- *	0 = no volume serial number
+ *  compiles the list of volume IDs
+ *    returns volume ID on success
+ *	0 = no volume ID
  *	-1 = error
  */
 unsigned long register_bios(int bios, int device)
@@ -583,7 +586,10 @@ unsigned long register_bios(int bios, int device)
 	inited = 1;
 	srand(time(NULL));
     }
+#else
+    inited = 1;
 #endif
+    if (!do_md_install && cfg_get_flag(cf_options, "static-bios-codes")) return 0;
     
     if (verbose>=4) {
 	printf("registering bios=0x%02X  device=0x%04X\n", bios, device);
@@ -609,13 +615,13 @@ unsigned long register_bios(int bios, int device)
 		i = device % PRIME + SMALL_PRIME;
 		while(i--) serial = rand();
 		for (; i<5 && !serial_valid(serial, disk_bios); i++) serial = rand();
-		if (!serial_valid(serial, disk_bios)) die("serial number generation error");
+		if (!serial_valid(serial, disk_bios)) die("Volume ID generation error");
 
 		*(long*)&buff.sector[PART_TABLE_OFFSET-6] = serial;
 		if (*(short*)&buff.sector[PART_TABLE_OFFSET - 2] == 0)
 		    *(short*)&buff.sector[PART_TABLE_OFFSET - 2] = MAGIC_SERIAL;
 		if (verbose)
-		    printf("Assigning new serial number to (%04X) '%s'  S/N %08X\n",
+		    printf("Assigning new Volume ID to (%04X) '%s'  ID = %08X\n",
 		    			device, dev.name, (int)serial);
 		if (!test) {
 		    i = lseek(fd, 0L, SEEK_SET);
@@ -653,7 +659,7 @@ unsigned long register_bios(int bios, int device)
 	    	bios|0x80, dev.name, device_code[bios], deva.name, device);
 	}
 	if (verbose>=3) {
-	    printf("Using serial number %08X on bios %02X\n", (int)serial, bios+0x80);
+	    printf("Using Volume ID %08X on bios %02X\n", (int)serial, bios+0x80);
 	}
     }
     else if (bios>=0 && bios <=3) serial = 0;
@@ -690,11 +696,11 @@ static int volid_get_set(int device, int vol_in, int option)
     unsigned short word;
     
     if (!has_partitions(device) && (device & P_MASK(device)) )
-	die("volid set/get bad device %04X\n", device);
+	die("VolumeID set/get bad device %04X\n", device);
 	
     fd = dev_open(&dev, device, option ? O_RDWR : O_RDONLY);
     if (read(fd, &buf, sizeof(buf)) != sizeof(buf))
-	die("volid read error");
+	die("VolumeID read error: sector 0 of %s not readable", dev.name);
     if (option==ID_SET) {
 	make_backup(NULL, 0, &buf, device,
 					"master disk volume ID record");
@@ -716,6 +722,7 @@ static int volid_get_set(int device, int vol_in, int option)
 	}
     }
     dev_close(&dev);
+    sync(); /* critical that this be done here */
     
     return buf.boot.volume_id;
 }
@@ -756,6 +763,50 @@ struct VolumeMgmt {
 };
 
 
+#ifdef LCF_ATARAID
+/* return boolean if VM is a component drive of ATA */
+
+static int is_ata(struct VolumeMgmt *vm, struct VolumeMgmt *ata)
+{
+    int ata_fd;
+    DEVICE dev;
+    struct ata_array_info_s raid;
+    struct ata_version_s version;
+    int ret=0;
+    
+    if ((ata_fd=dev_open(&dev, ata->device, O_NOACCESS) ) < 0)
+	die("Unable to open %s", ata->name);
+
+    if (ioctl(ata_fd, ATA_RAID_VERSION, &version) < 0) ret = -1;
+    else if (version.major != ATA_MAJOR_VERSION ||
+		version.minor < ATA_MINOR_VERSION ) {
+	ret = -2;
+	fprintf(errstd, "Warning: "
+	    "ATAraid driver version (kernel %d.%d.%d) <> (lilo %d.%d.%d)",
+		version.major, version.minor, version.patchlevel,
+		ATA_MAJOR_VERSION, ATA_MINOR_VERSION, ATA_PATCHLEVEL );
+
+    }
+    else if (ioctl(ata_fd, ATA_GET_ARRAY_INFO, &raid) < 0) ret = -1;
+    else {
+	struct ata_disk_info_s disk;
+	int i;
+	for (i=0; ret==0 && i<raid.active_disks; i++) {
+	    disk.number = i;
+	    if (ioctl(ata_fd, ATA_GET_DISK_INFO, &disk) < 0) ret = -1;
+	    if (vm->device == MKDEV(disk.major, disk.minor)) ret = 1;
+	}
+    }
+    dev_close(&dev);
+
+    return ret;
+}
+#endif
+
+/* 
+   Returns 0 if not an NT, 2000, or XP boot disk (query user)
+   returns 1 if it is an NT ... boot disk; abort if fatal set
+*/
 static int winnt_check(struct VolumeMgmt *vm, int fatal)
 {
     int dev, ret;
@@ -786,11 +837,6 @@ static int winnt_check(struct VolumeMgmt *vm, int fatal)
 
 
 
-#if VERSION_MINOR>50 && DEBUG
-#define PARTITIONS "./devfs_partitions"
-#else
-#define PARTITIONS "/proc/partitions"
-#endif
 #define SORT(d) (MAJOR(d)==MAJOR_SD?MKDEV(MAJOR_SD_SORT,MINOR(d)):\
 		 MAJOR(d)==MAJOR_HPT370?MKDEV(MAJOR_HPT370_SORT,MINOR(d)):\
 		 (d))
@@ -801,23 +847,23 @@ int pf_hard_disk_scan(void)
 static int warned = 0, called = 0;
     char *line, *next, *name;
     int n, major, minor, i, ipart;
-    int dev, device, lastdevice, mask;
+    int dev, device, lastdevice, mask, bios;
     DEVICE Dev;
+    DT_ENTRY *walk;
     struct stat st;
-    FILE *pf;
     int duplicate = 0, invalid = 0, ret = 0, ntcaution = 0;
     int codes = 0;
 
 /* called from  raid_setup  &  from  geo_open */
 /* allow only 1 call */
-    if (called) return ret;
+    if (called || cfg_get_flag(cf_options,"static-bios-codes")) return ret;
     called = 1;
     memset(vm,0,sizeof(vm));	/* for consistency */
             
 #if 1
-    if ((pf = fopen(PARTITIONS, "r"))==NULL) {
+    if (!pp_fd  &&  (pp_fd = fopen(PARTITIONS, "r"))==NULL) {
 #else
-    if ((pf = fopen(PARTITIONS, "r"))==NULL || fetch()) {
+    if ((pp_fd = fopen(PARTITIONS, "r"))==NULL || fetch()) {
 #endif
 	fprintf(errstd, "Warning: '" PARTITIONS "' does not exist, disk scan bypassed\n");
 	return 1;
@@ -825,13 +871,13 @@ static int warned = 0, called = 0;
 
     lastdevice = n = 0;
     line = NULL;
-    while (!feof(pf)) {
+    while (!feof(pp_fd)) {
     	if (line) {
 	    free(line);
 	    line = NULL;
 	    n = 0;
     	}
-	if (getline(&line, &n, pf) <= 0) break;
+	if (getline(&line, &n, pp_fd) <= 0) break;
 
 	major = strtoul(line, &next, 10);
 	if (major==0 || line==next) continue;
@@ -889,6 +935,18 @@ static int warned = 0, called = 0;
 	dev = device & mask;	/* dev is the master device */
 	ipart = device & P_MASK(device); /* ipart is the partition number */
 	
+#if 1
+	for (walk=disktab; walk; walk=walk->next) {
+	    if (walk->device == dev) {
+		if (walk->heads == 0 /* inaccessible */) {
+		    if ((!nowarn || verbose>=2) && ipart==0 && !identify)  fprintf(errstd,
+			"Warning: bypassing VolumeID scan of drive flagged INACCESSIBLE:  %s\n", name);
+		    ipart = -1;	/* causes skip below */
+		}
+		break;
+	    }
+	}
+#endif
 	if (mask && ipart>0) {
 	    int found;
 	    long serial;
@@ -897,12 +955,11 @@ static int warned = 0, called = 0;
 		if (dev==vm[i].device) found = i+1;
 	    }
 	    if (!found) {
-		DT_ENTRY *walk;
 		DEVICE Dev2;
 		serial = volid_get_set(dev, 0, ID_GET);
 
 #if 0
-/* take care of uninitialized serial numbers with no fanfare */
+/* take care of uninitialized Volume IDs with no fanfare */
 		if (serial==0) {
 		    serial = volid_get_set(dev, new_serial(dev), ID_SET);
 		}
@@ -926,8 +983,7 @@ static int warned = 0, called = 0;
 
 		    for (walk = disktab; walk; walk = walk->next) {
 			if (walk->device == dev) {
-			    register int bios = walk->bios;
-			    
+			    bios = walk->bios;
 			    vm[ndevs].dt = walk;	/* record disktab link */
 			    if (bios >= 0x80 && bios <= DEV_MASK) {
 				vm[ndevs].bios.actual =
@@ -992,7 +1048,7 @@ static int warned = 0, called = 0;
     } /* while (!feof())  */
    
     if (line) free(line);
-    fclose(pf);
+    fclose(pp_fd);
 
 
 
@@ -1006,7 +1062,7 @@ static int warned = 0, called = 0;
 
 /* now sort the volumes into canonical order */
     {
-	int i,j;
+	int i,j,k;
 	
 	for (j=ndevs-1; j>0; j--)
 	for (i=0; i<j; i++)
@@ -1016,7 +1072,79 @@ static int warned = 0, called = 0;
 	    vm[i] = vm[i+1];
 	    vm[i+1] = temp;
 	}
-    }
+
+/* now automatically treat ATARAID devices as inaccessible */
+      for (k=0; k<ndevs; k++)
+#ifdef LCF_ATARAID
+      if (MAJOR(vm[k].device) == MAJOR_HPT370) {
+#if ATAKEYWORD
+	if (!cfg_get_flag(cf_options, "ataraid") ) {
+	    if (!nowarn) fprintf(errstd,
+	    	"Warning: ATARAID controller present, without \"ataraid\" keyword used.\n"
+	    	"    Underlying drives individually must be marked INACCESSIBLE.\n" );
+	} else {
+#endif
+	    for (j=k+1; j<ndevs; j++) {
+		if ((i=is_ata(&vm[j], &vm[k]))<0) {
+		    if (!nowarn && !identify) fprintf(errstd,
+			"Warning: (ATAraid driver) the kernel does not support underlying\n"
+			"    device inquiries.  Each underlying drive of  %s  must\n"
+			"    individually be marked INACCESSIBLE.\n", vm[k].name
+			);
+		    j=ndevs;	/* terminate loop */
+		}
+		else if (i) {
+		    if (!vm[j].dt) {
+			walk = alloc_t(DT_ENTRY);
+			walk->device = vm[j].device;
+			walk->cylinders = walk->heads = walk->sectors = walk->start = -1;
+			walk->next = disktab;
+			vm[j].dt = disktab = walk;
+#if BETA_TEST
+			if (verbose >= 4) printf("Allocated DT entry for device %04X  ptr=%08x\n", vm[j].device, (int)walk);
+#endif
+		    }
+
+		    if (vm[j].dt->heads != 0) {
+			vm[j].dt->heads = 0;
+			if (!nowarn || verbose>=1) fprintf(errstd,
+				"Warning: (ataraid) underlying device flagged INACCESSIBLE: %s\n",
+				vm[j].name);
+		    }
+		    --ndevs;
+		    if (!nowarn || verbose>=1)
+		    	fprintf(errstd,
+		    	"Warning: bypassing VolumeID check of underlying ATARAID drive:\n"
+		    	"\t%04X  %08X  %s\n",
+		    		vm[j].device, vm[j].vol_id.kernel, vm[j].name);
+		    for (i=j; i<ndevs; i++) vm[i] = vm[i+1];
+		    j = k;	/* proper place to resume scan */
+		}
+	    }
+#if ATAKEYWORD
+	}
+#endif
+      }
+#else
+	if (MAJOR(vm[k].device) == MAJOR_HPT370) {
+	    if (!nowarn) fprintf(errstd,
+	    	"Warning: ATARAID controller present; underlying drives individually\n"
+	    	"    must be marked INACCESSIBLE.\n" );
+#if BETA_TEST
+{
+	int ata_fd;
+	DEVICE dev;
+
+	if ((ata_fd=dev_open(&dev, vm[k].device, O_NOACCESS) ) < 0)
+	    die("Unable to open %s",vm[k].name);
+	if (ioctl(ata_fd, ATA_PRINT_RAID_DEBUG, NULL) < 0)
+	    die("ATA_PRINT_RAID_DEBUG not implemented");
+	dev_close(&dev);
+}
+#endif
+	}
+#endif
+    } /* end sort */
 
 	if (verbose>=3) {
 	    for (i=0; i<ndevs; i++)
@@ -1029,7 +1157,9 @@ static int warned = 0, called = 0;
     if (invalid)
     for (i=0; i<ndevs; i++)
     if (vm[i].flag & INVALID) {
-	winnt_check(&vm[i], 1);
+	if (ndevs>1) winnt_check(&vm[i], 1);
+	else if (vm[i].flag & NTCAUTION) break;
+	
 	dev = vm[i].device;
 	vm[i].vol_id.kernel = volid_get_set(dev, new_serial(dev), ID_SET);
 	vm[i].flag &= ~INVALID;
@@ -1070,9 +1200,6 @@ static int warned = 0, called = 0;
 	}
 
 
-{
-    DT_ENTRY *walk;
-    int bios;
 
     if (verbose>=2) printf("device codes (user assigned pf) = %X\n", codes);
 
@@ -1159,7 +1286,6 @@ static int warned = 0, called = 0;
 	fprintf(errstd,"Warning: BIOS device code 0x%02X is used (>0x%02X).  It indicates more disks\n"
 			"  than those represented in '/proc/partitions' having actual partitions.\n"
 			"  Booting results may be unpredictable.\n", bios+0x80, ndevs+0x80-1);
-}
 
 
 
