@@ -56,6 +56,17 @@ void dump_serial_nos(void);
 /* debugging dump of the Volume Serial number table */
 
 
+int serial_valid(unsigned long serial, int disk_bios);
+/* determine validity of serial number; liberally if disk_bios==0x80 */
 
 
+int pf_hard_disk_scan(void);
+/* scan /proc/partitions for devices & partitions */
+
+int yesno(char *prompt, int expect);
+/* prompt the user for a yes/no response */
+
+int new_serial(int device);
+/* generate a new volumeID */
+
 #endif
