@@ -1,6 +1,6 @@
 /* test4.c */
 /*
- Copyright 2001-2002 John Coffman.
+ Copyright 2001-2003 John Coffman.
  All rights reserved.
 
  Licensed under the terms contained in the file 'COPYING' in the LILO
@@ -10,7 +10,7 @@
 #include <bios.h>
 #include "../bdata.h"
 
-#define DISK_VERSION "2.1.1"
+#define DISK_VERSION "2.2.2"
 
 #if __MSDOS__==0
 #define putch bios_putc
@@ -226,12 +226,32 @@ void banner(char *version)
 {
 	printf(	"\n\n\n"
 ">>>> Disk Detection and Parameter Display <<<<\n\n\n"
-"Version %s, Copyright (C) 1999-2002  John Coffman <johninsd@san.rr.com>\n"
+"Version %s, Copyright (C) 1999-2003  John Coffman <johninsd@san.rr.com>\n"
 "Portions Copyright (C) 1996-2001 Robert de Bath, used with permission\n"
 "Re-use and redistribution rights set forth in the file \"COPYING\".\n\n",
 	 version);
 }
 
+static
+void testDX(void)
+{
+#if __MSDOS__==0
+   printf("Boot reported from DX = 0x%04x (boot device is 0x%02x in DL)\n", __argr.x.dx, __argr.h.dl);
+   if (__argr.h.dl == 0 || __argr.h.dl == 1) {
+   	printf("If you booted from the %s floppy drive, then this is correct.",
+   		__argr.h.dl ? "second" : "first");
+   } else if (__argr.h.dl >= 0x80 && __argr.h.dl <= 0x8f) {
+   	printf("If you booted from %s hard drive, then this is correct.",
+   		__argr.h.dl==0x80 ? "the first" :
+   		__argr.h.dl==0x81 ? "the second" : "a" );
+   } else {
+	printf("It looks like the BIOS failed to report the boot device in DL.\n");
+   }
+#endif
+}
+
+
+
 static
 int smsw(void)
 {
@@ -656,6 +676,34 @@ int get_cfg_info(void)
     return 1;
 }
 
+static
+int enable_refresh(void)
+{
+    pause();
+    printf(SPACER
+    		"Int 10h  Function 12h\t\t[VGA]\n"
+    		"Subfunction 36h\n"
+    		"Enable Screen Refresh\n\n"
+    		"Call With:\n    ");
+    reg.x.ax = 0x1200;
+    reg.x.bx = 0x0036;
+    reg.x.cx = 0;
+    reg.x.dx = 0x80;
+    print_regs(&reg);
+    int86(0x10, &reg, &oreg);
+    printf("\nReturns:\n    ");
+    print_regs(&oreg);
+    
+    printf("\n");
+    printf("Function is %ssupported.\n", oreg.h.al==0x12 ? "" : "NOT ");
+    if (oreg.x.dx != reg.x.dx || oreg.x.cx != reg.x.cx || oreg.x.si != reg.x.si
+    		|| oreg.x.di != reg.x.di)
+	printf("Error: Register(s) are not preserved.\n");
+    reg.x.dx = 0;
+    
+    return 1;
+}
+
 static
 int get_comb_code(void)
 {
@@ -1022,7 +1070,7 @@ void main(void)
 /**	atexit(pause);	**/
 	video_fix();	/* for Dumb DELL computers */
     }
-#if DEBUG>=1
+#if DEBUG>=1 && __MSDOS__==0
     printf("Beginning of '___cstartup'\n");
     print_regs(&__argr);
     printf("DS=%04x  ES=%04x  CS=%04x  SS=%04x  SP=%04x  BP=%04x\n",
@@ -1035,6 +1083,7 @@ void main(void)
 #endif
     banner(DISK_VERSION);
     v86test();
+    testDX();
 #if DEBUG>=2
     sizeit((long)40*2*9);  putch('\n');
     sizeit((long)80*2*15);  putch('\n');
@@ -1050,6 +1099,7 @@ void main(void)
     if (hma>1024L*1024L) mov_ext_mem();
     m = get_video_mode();
     if (m) m = get_cfg_info();
+    if (m) m = enable_refresh();
     if (m) m = get_comb_code();
     if (m) do_vesa();
 #if DEBUG>=3
