@@ -118,11 +118,12 @@ int get_std_headers(int fd,
 
     *lh = lh0;
     npal = 1 << (bmh->numBitPlanes * bmh->numBitsPerPlane);
+    colormax = npal - 1;
     if (npal > nelem(palette) )   return 3;
     for (i=0; i<npal; i++) {
 	if (read(fd, &palette[i], n) != n)  return -1;
 	if (n==sizeof(RGB2)) palette[i].null = 0;
-    } 
+    }
     if (*(long*)(fh->offsetToBits) == sizeof(BITMAPFILEHEADER) +
     		sizeof(BITMAPHEADER) + sizeof(BITMAPLILOHEADER) +
     		npal*sizeof(RGB) )  /* test will fail for OS/2 bitmaps */ {
