@@ -182,7 +182,10 @@ int has_partitions_beta(dev_t dev)
     if (
       major == MAJOR_HD || major == MAJOR_IDE2 ||
       major == MAJOR_IDE3 || major == MAJOR_IDE4 ||
-      major == MAJOR_IDE5 || major == MAJOR_IDE6 ||
+#ifdef MAJOR_IDE5
+      major == MAJOR_IDE5 || 
+#endif
+      major == MAJOR_IDE6 ||
       (major >= MAJOR_IDE7 && major <= MAJOR_IDE10) ||
       major == MAJOR_XT || major == MAJOR_ESDI || major == MAJOR_ACORN
       	) return 0xFFC0;	/* 6 bit partition mask */
@@ -194,7 +197,7 @@ int has_partitions_beta(dev_t dev)
       (major >= MAJOR_I2O && major <= MAJOR_I2O+7) ||
       (major >= MAJOR_SMART2 && major <= MAJOR_SMART2+7) ||
       (major >= MAJOR_CISS && major <= MAJOR_CISS+7) ||
-      major == MAJOR_FTL || major == MAJOR_NFTL
+      major == MAJOR_FTL || major == MAJOR_NFTL || major == MAJOR_DOC
         ) return 0xFFF0;	/* 4 bit partition mask */
 
     if ( major == MAJOR_IBM_iSER ||
@@ -205,23 +208,26 @@ int has_partitions_beta(dev_t dev)
 }
 #endif
 
-unsigned char max_partno[] = {
+unsigned char max_partno[256] = {
 /*
   0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
 */
   0,  0,  0, 63,  0,  0,  0,  0, 15,  0,  0,  0,  0, 63,  0,  0,   /*  0x   */
   0,  0,  0,  0,  0, 63, 63,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0, 63, 63,  0, 63,  0,  0,  0,  0,  0,  0,  0, 15,  0,  0,  0,
+#ifndef MAJOR_IDE5
+  7,  7,  7,  7,  7,  7,  7,  7,  0, 63,  0,  0, 15, 15, 15, 15,
+#else
   7,  7,  7,  7,  7,  7,  7,  7, 63, 63,  0,  0, 15, 15, 15, 15,
+#endif
 
   0, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,   /*  4x   */
  15, 15, 15, 15, 15, 15, 15, 15, 63, 63, 63, 63,  0, 15,  0,  0,
-  0,  0,  0,  0,  0, 15,  0,  0, 15, 15, 15, 15, 15, 15, 15, 15,
+  0,  0,  0,  0, 15, 15,  0,  0, 15, 15, 15, 15, 15, 15, 15, 15,
   7,  0, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
 /*
   0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
 */
-#if 0
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   /*  8x   */
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
@@ -234,7 +240,6 @@ unsigned char max_partno[] = {
 /*
   0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
 */
-#endif
 };
 
 int has_partitions(dev_t dev)
@@ -243,7 +248,7 @@ int has_partitions(dev_t dev)
    int ret;
    
    if (
-   	major < sizeof(max_partno) &&
+/***   	major < sizeof(max_partno) &&	***/
    	(ret=max_partno[major])
    					)  ret ^= 0xFFFF;
    else ret = 0;
@@ -256,7 +261,8 @@ void do_disk(void)
 {
     DT_ENTRY *entry,*walk;
     struct stat st;
-    char *disk,*bios,*sectors,*heads,*cylinders;
+    char *disk,*bios,*sectors,*heads,*cylinders,*maxpart;
+    int major;
 
     entry = alloc_t(DT_ENTRY);
     disk = cfg_get_strg(cf_options,"disk");
@@ -265,12 +271,20 @@ void do_disk(void)
     	(has_partitions(st.st_rdev) && (MINOR(st.st_rdev) & P_MASK(st.st_rdev))))
 		die(" '%s' is not a whole disk device",disk);
     entry->device = st.st_rdev;
+    major = MAJOR(st.st_rdev);
     cfg_init(cf_disk);
     (void) cfg_parse(cf_disk);
     bios = cfg_get_strg(cf_disk,"bios");
     sectors = cfg_get_strg(cf_disk,"sectors");
     heads = cfg_get_strg(cf_disk,"heads");
     cylinders = cfg_get_strg(cf_disk,"cylinders");
+    maxpart = cfg_get_strg(cf_disk,"max-partitions");
+    if (maxpart) {
+	int i = to_number(maxpart);
+	if (max_partno[major] && max_partno[major]!=i) die("Cannot alter 'max-partitions' for known disk  %s", disk);
+	max_partno[major] = i;
+	if (i!=7 && i!=15 && i!=63) die("disk=%s:  illegal value for max-partitions(%d)", disk, i);
+    }
     entry->bios = bios ? to_number(bios) : -1;
     if (!sectors && !heads) entry->sectors = entry->heads = -1;
     else if (!(sectors && heads))
@@ -488,7 +502,7 @@ void evms_bmap(struct evms_get_bmap_t *ebm)
 void geo_query_dev(GEOMETRY *geo,int device,int all)
 {
     DEVICE dev;
-    int fd,get_all;
+    int fd,get_all,major;
     struct floppy_struct fdprm;
     struct hd_geometry hdprm;
 
@@ -514,7 +528,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	geo->start = 0;
 	geo->device = -1;
     }
-    switch (MAJOR(device)) {
+    switch ((major=MAJOR(device))) {
 	case MAJOR_FD:
 	    geo->device = device & 3;
 	    if (!get_all) break;
@@ -530,7 +544,9 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_IDE2:
 	case MAJOR_IDE3:
 	case MAJOR_IDE4:
+#ifdef MAJOR_IDE5
 	case MAJOR_IDE5:
+#endif
 	case MAJOR_IDE6:
 	case MAJOR_IDE7:
 	case MAJOR_IDE8:
@@ -539,6 +555,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_ESDI:
 	case MAJOR_XT:
 	case MAJOR_ACORN:
+	MASK63:
 	    geo->device = 0x80 + (MINOR(device) >> 6) +
 		    (MAJOR(device) == MAJOR_HD ? 0 : last_dev(MAJOR_HD,64));
 	    if (!get_all) break;
@@ -558,6 +575,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_SD6:
 	case MAJOR_SD7:
 	case MAJOR_SD8:
+	MASK15:
 	    geo->device = 0x80 + last_dev(MAJOR_HD,64) + (MINOR(device) >> 4);
 	    if (!get_all) break;
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
@@ -580,6 +598,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_DAC960+6:
 	case MAJOR_DAC960+7:
 	case MAJOR_IBM_iSER:
+	MASK7:
 	    geo->device = 0x80 + last_dev(MAJOR_HD,64) + (MINOR(device) >> 3);
 	    if (!get_all) break;
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
@@ -601,6 +620,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_EXPR+3:
 	case MAJOR_FTL:
 	case MAJOR_NFTL:
+	case MAJOR_DOC:
 	case MAJOR_SMART2+0:
 	case MAJOR_SMART2+1:
 	case MAJOR_SMART2+2:
@@ -640,6 +660,10 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    break;
 
 	default:
+	    if (max_partno[major] == 63)  goto MASK63;
+	    if (max_partno[major] == 15)  goto MASK15;
+	    if (max_partno[major] == 7)   goto MASK7;
+	    
 	    die("Sorry, don't know how to handle device 0x%04x",device);
     }
     if (get_all) dev_close(&dev);
@@ -669,7 +693,9 @@ int is_first(int device)
 	case MAJOR_IDE2:
 	case MAJOR_IDE3:
 	case MAJOR_IDE4:
+#ifdef MAJOR_IDE5
 	case MAJOR_IDE5:
+#endif
 	case MAJOR_IDE6:
 	case MAJOR_IDE7:
 	case MAJOR_IDE8:
@@ -694,6 +720,7 @@ int is_first(int device)
 	case MAJOR_EXPR+2:
 	case MAJOR_EXPR+3:
 	case MAJOR_NFTL:
+	case MAJOR_DOC:
 	case MAJOR_SMART2+0:
 	case MAJOR_SMART2+1:
 	case MAJOR_SMART2+2:
@@ -791,6 +818,7 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     if (MAJOR(device) == MD_MAJOR) {
         char mdxxx[16];
 	int md_fd, pass;
+	struct md_version md_version_info;
 	md_array_info_t md_array_info;
 	md_disk_info_t md_disk_info;
 	int raid_limit;
@@ -802,10 +830,18 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	    if ((md_fd=open(mdxxx,O_NOACCESS)) < 0)
 		die("Unable to open %s", mdxxx);
 	}
+	if (ioctl(md_fd,RAID_VERSION,&md_version_info) < 0)
+	    die("Unable to get RAID version on %s", mdxxx);
+	if (md_version_info.major > 0)
+	    die("Raid major versions > 0 are not supported");
+	if (md_version_info.minor < 90)
+	    die("Raid versions < 0.90 are not supported");
+	
 	if (ioctl(md_fd,GET_ARRAY_INFO,&md_array_info) < 0)
 	    die("Unable to get RAID info on %s", mdxxx);
-	if ((md_array_info.major_version == 0) && (md_array_info.minor_version < 90))
-	    die("Raid versions < 0.90 are not supported");
+	if ((md_array_info.major_version != md_version_info.major) &&
+		(md_array_info.minor_version != md_version_info.minor))
+	    die("Inconsistent Raid version information on %s", mdxxx);
 	if (md_array_info.level != 1)
 	    die("Only RAID1 devices are supported for boot images");
 	raid_limit = md_array_info.raid_disks + md_array_info.spare_disks;
@@ -1174,12 +1210,22 @@ int geo_find(GEOMETRY *geo,SECTOR_ADDR addr)
     struct stat st;
     int i;
 
+#if DEBUG_NEW
+    if (verbose>=2) {
+	printf("Find:  AL=%02x  CX=%04x  DX=%04x  LBA=%d\n", (int)addr.num_sect,
+		addr.sector + (addr.track<<8),
+		addr.device + (addr.head<<8),
+		addr.sector + (addr.track<<8) + (addr.head<<16) +
+		(addr.device&(LBA32_FLAG|LBA32_NOCOUNT)?addr.num_sect<<24:0) );
+    }
+#endif
     if (fstat(geo->fd,&st) < 0) return 0;
     geo_get(geo,st.st_dev,-1,1);
     for (i = 0; i < (st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE; i++)
 	if (geo_comp_addr(geo,i*SECTOR_SIZE,&here))
 	    if (here.sector == addr.sector && here.track == addr.track &&
-	      here.device == addr.device && here.head == addr.head) {
+	      here.device == addr.device && here.head == addr.head &&
+	      here.num_sect == addr.num_sect ) {
 		if (lseek(geo->fd,i*SECTOR_SIZE,SEEK_SET) < 0) return 0;
 		else return 1;
 	    }
