@@ -20,21 +20,24 @@ source directory.
 #include "common.h"
 #include "cfg.h"
 
-
-static char *identify,*opt;
+char *identify;
+static char *opt;
+static char *first, *dflt;
+static int idefault;
 
 
 static void do_identify(char *var,char type)
 {
-    char *label,*path,*alias,*initrd,*keytab;
-    int image, ramdisk,kt;
+    char *label,*path,*alias,*initrd,*keytab,*appstr,*dtem;
+    int image, ramdisk, kt, append;
 
 #if 1
     image = !!strchr(opt,'i');
     ramdisk = !!strchr(opt,'r');
     kt = !!strchr(opt,'k');
-    if (opt && !image && !ramdisk && !kt) exit(1);
-    if (!opt) image = 1;
+    append = !!strchr(opt,'a');
+    if (opt && !image && !ramdisk && !kt && !append && !idefault) exit(1);
+/*    if (!opt) image = 1; */
 #else
     image = ramdisk = 1;
     printf("do_identify:  opt=\"%s\"\n", opt);
@@ -44,10 +47,29 @@ static void do_identify(char *var,char type)
     if (label) label++;
     if (cfg_get_strg(cf_all,"label")) label = cfg_get_strg(cf_all,"label");
     else if (!label) label = path;
+
+    if (!first) first = stralloc(label);
+    
     alias = cfg_get_strg(cf_all,"alias");
+    dtem = cfg_get_strg(cf_options,"default");
+
+    if (verbose>=2) printf("identify: dtem=%s  label=%s\n", dtem, label);
+#ifdef LCF_IGNORECASE
+    if (dtem && (!strcasecmp(label,dtem) || (alias && !strcasecmp(alias,dtem)))) {
+#else
+    if (dtem && (!strcmp(label,dtem) || (alias && !strcmp(alias,dtem)))) {
+#endif
+	if (verbose>=2) printf("setting  dflt\n");
+	dflt = dtem;
+    }
+
     initrd = cfg_get_strg(cf_kernel,"initrd");
+    if (!initrd) initrd = cfg_get_strg(cf_options,"initrd");
     keytab = cfg_get_strg(cf_options,"keytable");
     if (!keytab) keytab="us.ktl";
+    appstr = cfg_get_strg(cf_kernel,"append");
+    if (!appstr) appstr = cfg_get_strg(cf_options,"append");
+
 #ifdef LCF_IGNORECASE
     if (!strcasecmp(label,identify) || (alias && !strcasecmp(alias,identify))) {
 #else
@@ -56,6 +78,8 @@ static void do_identify(char *var,char type)
 	if (image) printf("%s\n",path);
 	if (ramdisk) printf("%s\n",initrd?initrd:"No initial ramdisk specified");
 	if (kt) printf("%s\n",keytab);
+	if (append) printf("%s\n",appstr?appstr:"No append= was specified");
+	if (idefault) printf("%s\n", dflt ? dflt : first);
 	exit(0);
     }
 }
@@ -84,8 +108,15 @@ void identify_image(char *label,char *options)
 {
     identify = label;
     opt = options;
+    if (verbose>=2) printf("identify_image: id='%s' opt='%s'\n", label, options);
+    idefault = !!strchr(opt,'D');
+    if (idefault) identify = "";
     cfg_init(cf_identify);
     if (cfg_parse(cf_identify)) cfg_error("Syntax error");
+    if (idefault && first) {
+	printf("%s\n", dflt ? dflt : first);
+	exit(0);
+    }
     fprintf(errstd,"No image found for \"%s\"\n",label);
     exit(1);
 }
