@@ -1,7 +1,7 @@
 /* lilo.c  -  LILO command-line parameter processing */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -53,7 +53,7 @@ static void show_other(int fd)
     const unsigned char *prtmap;
 
     if (read(fd,buf,sizeof(buf)) != sizeof(buf))
-	die("Read on map file failed (access conflict ?)");
+	die("Read on map file failed (access conflict ?) 1");
     if (!strncmp(buf[0].par_c.signature-4,"LILO",4)) {
 	printf("    Pre-21 signature (0x%02x,0x%02x,0x%02x,0x%02x)\n",
 	  buf[0].par_c.signature[0],buf[0].par_c.signature[1],
@@ -264,12 +264,12 @@ static void show_images(char *map_file)
 		}
 		if (read(fd,addr,4*sizeof(SECTOR_ADDR)) !=
 		  4*sizeof(SECTOR_ADDR))
-			die("Read on map file failed (access conflict ?)");
+			die("Read on map file failed (access conflict ?) 2");
 		if (!geo_find(&geo,addr[0]))
 		    printf("    Fallback sector not found\n");
 		else {
 		    if (read(fd,buffer,SECTOR_SIZE) != SECTOR_SIZE)
-			die("Read on map file failed (access conflict ?)");
+			die("Read on map file failed (access conflict ?) 3");
 		    if (*(unsigned short *) buffer != DC_MAGIC)
 			printf("    No fallback\n");
 		    else printf("    Fallback: \"%s\"\n",buffer+2);
@@ -282,7 +282,7 @@ static void show_images(char *map_file)
 			printf("    Options sector not found\n");
 		    else {
 			if (read(fd,buffer,SECTOR_SIZE) != SECTOR_SIZE)
-			    die("Read on map file failed (access conflict ?)");
+			    die("Read on map file failed (access conflict ?) 4");
 			if (*buffer) printf("    Options: \"%s\"\n",buffer);
 			else printf("    No options\n");
 		    }
@@ -343,10 +343,10 @@ static void usage(char *name)
 
 int main(int argc,char **argv)
 {
-    char *name,*reboot_arg,*identify,*ident_opt,*new_root;
+    char *name,*reboot_arg,*ident_opt,*new_root;
     char *tell_param, *uninst_dev, *param, *act1, *act2, ch;
 static char *bitmap_file;
-    int query,more,version,uninstall,validate,activate,instmbr,geom;
+    int more,version,uninstall,validate,activate,instmbr,geom;
     struct stat st;
     int fd, temp, tell_early=0;
     long raid_offset;
@@ -360,7 +360,7 @@ static char *bitmap_file;
     act1 = act2 = tell_param = 
 	    reboot_arg = identify = ident_opt = new_root = uninst_dev = NULL;
     do_md_install = zflag =
-	    query = version = uninstall = validate = activate = instmbr = 0;
+	    version = uninstall = validate = activate = instmbr = 0;
     verbose = -1;
     name = *argv;
     argc--;
@@ -631,10 +631,14 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 
 
+    printf("\n");
     for (i=0; i<256; i++) {
-	k += ( has_partitions_beta(MKDEV(i,0)) != has_partitions(MKDEV(i,0)) );
+	if ( has_partitions_beta(MKDEV(i,0)) != has_partitions(MKDEV(i,0)) ) {
+	    printf("Major device = %d is not configured.\n", i);
+	    k ++;
+	}
     }
-    printf("\nk = %d (should be 0)\n", k);
+    printf("k = %d (should be 0)\n\n", k);
 }
 #endif
 
@@ -666,6 +670,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
     if (argc) usage(name);
     if (!new_root) new_root = getenv("ROOT");
     if (new_root && *new_root) {
+	pp_fd = fopen(PARTITIONS, "r");
 	if (chroot(new_root) < 0) die("chroot %s: %s",new_root,strerror(errno));
 	if (chdir("/dev") < 0 && nowarn==0)
 	    fprintf(errstd, "Warning: root at %s has no /dev directory\n", new_root);
@@ -682,7 +687,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	    return 0;
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
-	       "Development beyond version 21 Copyright (C) 1999-2003 John Coffman\n"
+	       "Development beyond version 21 Copyright (C) 1999-2004 John Coffman\n"
 	       );
         if (verbose>0) printf("Released %s%s and compiled at %s on %s%s\n",
 	        VERSION_DATE, comma ? "," : "", __TIME__, __DATE__, semi);
