@@ -40,6 +40,73 @@ struct disk_param {
 
 /* the following structures are created by the biosdata.S codes */
 
+typedef
+struct Equip {
+   unsigned short equipment;
+   unsigned short mem;
+#if PROBE_VERSION >= 5
+   unsigned short boot_dx;
+#endif
+} equip_t;
+
+
+/* BD_GET_VIDEO >= 1 */
+typedef
+struct Video1 {
+   struct {
+      unsigned char  al;
+      unsigned char  ah;
+      unsigned char  bl;
+      unsigned char  bh;
+   } vid0F;
+} video_t1;
+
+
+/* BD_GET_VIDEO >= 2  */
+typedef
+struct Video2 {
+   struct {
+      unsigned short ax;
+      unsigned char  bl;
+      unsigned char  bh;
+   } vid12;
+   struct {
+      unsigned char  al;
+      unsigned char  ah;
+      unsigned short bx;
+   } vid1A;
+} video_t2;
+
+/* BD_GET_VIDEO >=2 extension for PROBE_VERSION 5 */
+typedef
+struct Video25 {
+   struct {
+      unsigned short ax;
+      unsigned short cx;
+      unsigned short dx;
+      unsigned short bp;
+   } vid36;
+} video_t25;
+
+
+/* BD_GET_VIDEO >= 3  */
+typedef
+struct Video3 {
+   struct {
+      unsigned short ax;
+      unsigned char  sig[4];
+   } vid4F00;
+   struct {
+      unsigned short ax;
+      unsigned short bits;
+   } vid101;
+   struct {
+      unsigned short ax;
+      unsigned short bits;
+   } vid103;
+} video_t3;
+
+
 typedef
 struct Video {
    unsigned short equipment;
@@ -54,13 +121,11 @@ struct Video {
    } vid0F;
 
 /* BD_GET_VIDEO >= 2  */
-#if 1
    struct {
       unsigned short ax;
       unsigned char  bl;
       unsigned char  bh;
    } vid12;
-#endif
    struct {
       unsigned char  al;
       unsigned char  ah;
@@ -83,6 +148,7 @@ struct Video {
    } vid103;
 } video_t;
 
+
 typedef
 struct Floppy {
    struct {
@@ -101,6 +167,7 @@ struct Floppy {
    } fn08;
 } floppy_t;
 
+
 typedef
 struct Hard {
    struct {
@@ -148,6 +215,10 @@ struct Edd {
 enum {VIDEO_UNKNOWN, VIDEO_MDA, VIDEO_CGA, VIDEO_EGA, VIDEO_MCGA,
 	VIDEO_VGA, VIDEO_VESA, VIDEO_VESA_800};
 
+int fetch(void);
+
+int purge(void);
+
 void probe_tell (char *cmd);
 
 int bios_max_devs(void);
@@ -156,5 +227,7 @@ int bios_device(GEOMETRY *geo, int device);
 
 int get_video(void);	/* return -1 on error, or adapter type [0..7] */
 
+void check_bios(void);	/* set up bios_passes_dl */
+
 #endif
 /* end probe.h */
