@@ -32,7 +32,6 @@ source directory.
 #include "md-int.h"
 
 static int lowest;
-static md_array_info_t md_array_info;
 static DT_ENTRY *md_disk;
 static DT_ENTRY *disk;
 static unsigned long raid_base, raid_offset[MAX_RAID];
@@ -92,7 +91,9 @@ long raid_setup(void)
     int pass, mask;
     struct stat st;
     int md_fd;
+    struct md_version md_version_info;
     md_disk_info_t md_disk_info;
+    md_array_info_t md_array_info;
     GEOMETRY geo;
     DEVICE dev;
     char *extrap;
@@ -136,10 +137,18 @@ long raid_setup(void)
 
 	boot_dev_nr = st.st_rdev;	/* set this very early */
 
+	if (ioctl(md_fd,RAID_VERSION,&md_version_info) < 0)
+	    die("Unable to get RAID version on %s", boot);
+	if (md_version_info.major > 0)
+	    die("Raid major versions > 0 are not supported");
+	if (md_version_info.minor < 90)
+	    die("Raid versions < 0.90 are not supported");
+	
 	if (ioctl(md_fd,GET_ARRAY_INFO,&md_array_info) < 0)
 	    die("Unable to get RAID info on %s",boot);
-	if ((md_array_info.major_version == 0) && (md_array_info.minor_version < 90))
-	    die("Raid versions < 0.90 are not supported");
+	if ((md_array_info.major_version != md_version_info.major) &&
+		(md_array_info.minor_version != md_version_info.minor))
+	    die("Inconsistent Raid version information on %s", boot);
 	if (md_array_info.level != 1)
 	    die("Only RAID1 devices are supported as boot devices");
 	if (!linear && !lba32) {
@@ -427,11 +436,13 @@ long raid_setup(void)
 	       "Using BIOS device code 0x%02X for RAID boot blocks\n",
 	    	                            md_disk->bios);
 	}
+
+#if 0
 	if ( mask &&  ( extra == X_NONE ||
 			(extra == X_AUTO  &&  autocount == 0) ) )  {
-	     
 	    if (bios_passes_dl > DL_BAD) bios_passes_dl = DL_GOOD;
 	}
+#endif
 	
 	if (!nowarn  &&  bios_passes_dl==DL_GOOD  &&  !(extra == X_MBR_ONLY || extra == X_MBR))
 	    fprintf(errstd, "Warning: Boot sector on  %s  will depend upon the BIOS device code\n"
