@@ -7,7 +7,7 @@ Licensed under the terms contained in the file 'COPYING' in the
 source directory.
 
 */
-int do_md_install, ndisk;
+int do_md_install, ndisk, md_bios;
 
 long raid_setup(void);
 void raid_final(void);
